import os
import sys
from subprocess import Popen, PIPE, STDOUT

USAGE = """USAGE
  rf-browser [command]

AVAILABLE COMMANDS
  init  Install required nodejs dependencies
"""


def run():
    if len(sys.argv) < 2:
        print(USAGE)
        sys.exit(1)
    cmd = sys.argv[1]
    if cmd == "init":
        print("installing node dependencies...")
        installation_dir = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), "wrapper"
        )
        if not os.path.isfile(os.path.join(installation_dir, "package.json")):
            print(
                "Directory needs to contain package.json "
                + "and have write permissions to succesfully install all dependencies. "
                + "\nPrinting contents:"
            )
            for root, dirs, files in os.walk(installation_dir):
                level = root.replace(installation_dir, "").count(os.sep)
                indent = " " * 4 * (level)
                print("{}{}/".format(indent, os.path.basename(root)))
                subindent = " " * 4 * (level + 1)
                for f in files:
                    print("{}{}".format(subindent, f))
            raise RuntimeError("Could not find robotframework-browser's package.json")

        print("installing rfbrowser node dependencies at {}".format(installation_dir))
        process = Popen(
            "npm install --production",
            shell=True,
            cwd=installation_dir,
            stdout=PIPE,
            stderr=STDOUT,
        )
        if process.stdout is None:
            raise RuntimeError(
                "Problem installing node dependencies."
                + "Node process returned with exit status {}".format(process.returncode)
            )
        for line in process.stdout:
            print(line.decode("utf-8"))
        print("rfbrowser init completed")
    else:
        print(f"Invalid command `{cmd}`")
        print(USAGE)



from robot.api import logger  # type: ignore
from robotlibcore import keyword  # type: ignore

from ..generated import playwright_pb2


class Input:
    def __init__(self, library):
        self.library = library

    @property
    def playwright(self):
        return self.library.playwright

    @keyword
    def input_text(self, selector: str, text: str):
        """ Types the given ``text`` into the text field identified by ``selector`` """
        with self.playwright.grpc_channel() as stub:
            response = stub.InputText(
                playwright_pb2.inputTextRequest(input=text, selector=selector)
            )
            logger.info(response.log)

    @keyword
    def click_button(self, selector: str):
        """ Clicks the button identified by ``selector``. """
        with self.playwright.grpc_channel() as stub:
            response = stub.ClickButton(
                playwright_pb2.selectorRequest(selector=selector)
            )
            logger.info(response.log)

    # TODO: Should this be select_checkbox for consistency with SL or check_checkbox
    # for clarity and PW consistency?
    @keyword
    def check_checkbox(self, selector: str):
        """ Checks the checkbox identified by ``selector``.
            If already checked does nothing
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.CheckCheckbox(
                playwright_pb2.selectorRequest(selector=selector)
            )
            logger.info(response.log)

    @keyword
    def uncheck_checkbox(self, selector: str):
        """ Unchecks the checkbox identified by ``selector``.
            If not checked does nothing
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.UncheckCheckbox(
                playwright_pb2.selectorRequest(selector=selector)
            )
            logger.info(response.log)



__version__ = "0.2.2"

import re
import os

from robot.api import logger  # type: ignore
from robot.libraries.BuiltIn import BuiltIn, EXECUTION_CONTEXTS  # type: ignore
from robotlibcore import DynamicCore  # type: ignore

from .keywords import Validation, Control, Input
from .playwright import Playwright


class Browser(DynamicCore):
    """Browser library is a web testing library for Robot Framework.

    This documents explains how to use keywords provided by the Browser
    library. For information about installation, support, and more, please
    visit the
    [https://github.com/MarketSquare/robotframework-playwright|project pages].
    For more information about Robot Framework, see http://robotframework.org.

    Browser library uses
    [https://github.com/microsoft/playwright|Playwright Node module]
    to automate [https://www.chromium.org/Home|Chromium],
    [https://www.mozilla.org/en-US/firefox/new/|Firefox]
    and [https://webkit.org/|WebKit] with a single library.

    == Table of contents ==

    %TOC%

    = Locating elements =

    All keywords in Browser library that need to interact with an element
    on a web page take an argument typically named ``selector`` that specifies
    how to find the element.

    == Locator syntax ==

    Browser library supports same selector strategies as the underlying
    Playwright node module: xpath, css, id and text. The strategy can either
    be explicitly specified with a prefix or the strategy can be implicit.

    === Implicit selector strategy ===
    == !!! THIS ISN'T IMPLEMENTED YET !!! ==

    The default selector strategy is `css`. If selector does not contain
    one of the know selector strategies, `css`, `xpath`, `id` or `text` it is
    assumed to contain css selector. Also `selectors` starting with `//`
    considered as xpath selectors.

    Examples:

    | `Click` | span > button | # Use css selector strategy the element.   |
    | `Click` | //span/button | # Use xpath selector strategy the element. |

    === Explicit selector strategy ===

    The explicit selector strategy is specified with a prefix using syntax
    ``strategy=value``. Spaces around the separator are ignored, so
    ``css=foo``, ``css= foo`` and ``css = foo`` are all equivalent.


    Selector strategies that are supported by default are listed in the table
    below.

    | = Strategy = |     = Match based on =     |         = Example =            |
    | css          | CSS selector.              | ``css=div#example``            |
    | xpath        | XPath expression.          | ``xpath=//div[@id="example"]`` |
    | text         | Browser text engine.    | ``text=Login``                 |
    """

    ROBOT_LIBRARY_VERSION = __version__
    ROBOT_LISTENER_API_VERSION = 2
    ROBOT_LIBRARY_LISTENER: "Browser"
    ROBOT_LIBRARY_SCOPE = "GLOBAL"
    SUPPORTED_BROWSERS = ["chrome", "firefox", "webkit"]

    def __init__(self):
        self.ROBOT_LIBRARY_LISTENER = self
        self.browser_control = Control(self)
        libraries = [
            Validation(self),
            self.browser_control,
            Input(self),
        ]
        self.playwright = Playwright()
        DynamicCore.__init__(self, libraries)

    @property
    def outputdir(self):
        return BuiltIn().get_variable_value("${OUTPUTDIR}")

    def _close(self):
        self.playwright.close()

    def run_keyword(self, name, args, kwargs):
        try:
            return DynamicCore.run_keyword(self, name, args, kwargs)
        except AssertionError as e:
            self.test_error()
            raise e

    def start_keyword(self, name, attrs):
        """Take screenshot of tests that have failed due to timeout.

        This can be done with BuiltIn keyword `Run Keyword If Timeout
        Occurred`, but the problem there is that you have to remember to
        put it into your Suite/Test Teardown. Since taking screenshot is
        the most obvious thing to do on failure, let's do it automatically.

        This cannot be implemented as a `end_test` listener method, since at
        that time, the teardown has already been executed and browser may have
        been closed already. This implementation will take the screenshot
        before the teardown begins to execute.
        """
        if attrs["type"] == "Teardown":
            timeout_pattern = "Test timeout .* exceeded."
            test = EXECUTION_CONTEXTS.current.test
            if test.status == "FAIL" and re.match(timeout_pattern, test.message):
                self.browser_control.take_page_screenshot(
                    self.failure_screenshot_path(test.name)
                )

    def test_error(self):
        """Sends screenshot command to Playwright.

        Only works during testing since this uses robot's outputdir for output.
        """
        on_failure_keyword = "take page screenshot"
        try:
            test_name = BuiltIn().get_variable_value("${TEST NAME}")
            path = self.failure_screenshot_path(test_name)
            logger.info(f"Running `{on_failure_keyword}` with arguments `{path}`")
            BuiltIn().run_keyword(on_failure_keyword, path)
        except Exception as err:
            logger.error(
                f"Keyword '{on_failure_keyword}' could not be run on failure: {err}"
            )

    def failure_screenshot_path(self, test_name):
        return os.path.join(
            BuiltIn().get_variable_value("${OUTPUTDIR}"),
            test_name.replace(" ", "_") + "_FAILURE_SCREENSHOT",
        ).replace("\\", "\\\\")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: playwright.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='playwright.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x10playwright.proto\"\x07\n\x05\x45mpty\"!\n\x11screenshotRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\"2\n\x12openBrowserRequest\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x0f\n\x07\x62rowser\x18\x02 \x01(\t\"\x1a\n\x0bgoToRequest\x12\x0b\n\x03url\x18\x01 \x01(\t\"3\n\x10inputTextRequest\x12\r\n\x05input\x18\x01 \x01(\t\x12\x10\n\x08selector\x18\x02 \x01(\t\";\n\x15getDomPropertyRequest\x12\x10\n\x08property\x18\x01 \x01(\t\x12\x10\n\x08selector\x18\x02 \x01(\t\"#\n\x0fselectorRequest\x12\x10\n\x08selector\x18\x01 \x01(\t\"h\n\x08Response\x1a\x14\n\x05\x45mpty\x12\x0b\n\x03log\x18\x01 \x01(\t\x1a#\n\x06String\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0c\n\x04\x62ody\x18\x02 \x01(\t\x1a!\n\x04\x42ool\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0c\n\x04\x62ody\x18\x02 \x01(\x08\x32\xc8\x05\n\nPlaywright\x12\x33\n\nScreenshot\x12\x12.screenshotRequest\x1a\x0f.Response.Empty\"\x00\x12\x35\n\x0bOpenBrowser\x12\x13.openBrowserRequest\x1a\x0f.Response.Empty\"\x00\x12)\n\x0c\x43loseBrowser\x12\x06.Empty\x1a\x0f.Response.Empty\"\x00\x12\'\n\x04GoTo\x12\x0c.goToRequest\x1a\x0f.Response.Empty\"\x00\x12&\n\x08GetTitle\x12\x06.Empty\x1a\x10.Response.String\"\x00\x12\x31\n\tInputText\x12\x11.inputTextRequest\x1a\x0f.Response.Empty\"\x00\x12<\n\x0eGetDomProperty\x12\x16.getDomPropertyRequest\x1a\x10.Response.String\"\x00\x12;\n\x0fGetBoolProperty\x12\x16.getDomPropertyRequest\x1a\x0e.Response.Bool\"\x00\x12\x36\n\x0eGetTextContent\x12\x10.selectorRequest\x1a\x10.Response.String\"\x00\x12$\n\x06GetUrl\x12\x06.Empty\x1a\x10.Response.String\"\x00\x12\x32\n\x0b\x43lickButton\x12\x10.selectorRequest\x1a\x0f.Response.Empty\"\x00\x12\x34\n\rCheckCheckbox\x12\x10.selectorRequest\x1a\x0f.Response.Empty\"\x00\x12\x36\n\x0fUncheckCheckbox\x12\x10.selectorRequest\x1a\x0f.Response.Empty\"\x00\x12$\n\x06Health\x12\x06.Empty\x1a\x10.Response.String\"\x00\x62\x06proto3'
)




_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20,
  serialized_end=27,
)


_SCREENSHOTREQUEST = _descriptor.Descriptor(
  name='screenshotRequest',
  full_name='screenshotRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='screenshotRequest.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=29,
  serialized_end=62,
)


_OPENBROWSERREQUEST = _descriptor.Descriptor(
  name='openBrowserRequest',
  full_name='openBrowserRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='url', full_name='openBrowserRequest.url', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='browser', full_name='openBrowserRequest.browser', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=64,
  serialized_end=114,
)


_GOTOREQUEST = _descriptor.Descriptor(
  name='goToRequest',
  full_name='goToRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='url', full_name='goToRequest.url', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=116,
  serialized_end=142,
)


_INPUTTEXTREQUEST = _descriptor.Descriptor(
  name='inputTextRequest',
  full_name='inputTextRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='input', full_name='inputTextRequest.input', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='selector', full_name='inputTextRequest.selector', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=144,
  serialized_end=195,
)


_GETDOMPROPERTYREQUEST = _descriptor.Descriptor(
  name='getDomPropertyRequest',
  full_name='getDomPropertyRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='property', full_name='getDomPropertyRequest.property', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='selector', full_name='getDomPropertyRequest.selector', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=197,
  serialized_end=256,
)


_SELECTORREQUEST = _descriptor.Descriptor(
  name='selectorRequest',
  full_name='selectorRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='selectorRequest.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=258,
  serialized_end=293,
)


_RESPONSE_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='Response.Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='log', full_name='Response.Empty.log', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=307,
  serialized_end=327,
)

_RESPONSE_STRING = _descriptor.Descriptor(
  name='String',
  full_name='Response.String',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='log', full_name='Response.String.log', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='body', full_name='Response.String.body', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=329,
  serialized_end=364,
)

_RESPONSE_BOOL = _descriptor.Descriptor(
  name='Bool',
  full_name='Response.Bool',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='log', full_name='Response.Bool.log', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='body', full_name='Response.Bool.body', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=366,
  serialized_end=399,
)

_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RESPONSE_EMPTY, _RESPONSE_STRING, _RESPONSE_BOOL, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=295,
  serialized_end=399,
)

_RESPONSE_EMPTY.containing_type = _RESPONSE
_RESPONSE_STRING.containing_type = _RESPONSE
_RESPONSE_BOOL.containing_type = _RESPONSE
DESCRIPTOR.message_types_by_name['Empty'] = _EMPTY
DESCRIPTOR.message_types_by_name['screenshotRequest'] = _SCREENSHOTREQUEST
DESCRIPTOR.message_types_by_name['openBrowserRequest'] = _OPENBROWSERREQUEST
DESCRIPTOR.message_types_by_name['goToRequest'] = _GOTOREQUEST
DESCRIPTOR.message_types_by_name['inputTextRequest'] = _INPUTTEXTREQUEST
DESCRIPTOR.message_types_by_name['getDomPropertyRequest'] = _GETDOMPROPERTYREQUEST
DESCRIPTOR.message_types_by_name['selectorRequest'] = _SELECTORREQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Empty = _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), {
  'DESCRIPTOR' : _EMPTY,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:Empty)
  })
_sym_db.RegisterMessage(Empty)

screenshotRequest = _reflection.GeneratedProtocolMessageType('screenshotRequest', (_message.Message,), {
  'DESCRIPTOR' : _SCREENSHOTREQUEST,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:screenshotRequest)
  })
_sym_db.RegisterMessage(screenshotRequest)

openBrowserRequest = _reflection.GeneratedProtocolMessageType('openBrowserRequest', (_message.Message,), {
  'DESCRIPTOR' : _OPENBROWSERREQUEST,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:openBrowserRequest)
  })
_sym_db.RegisterMessage(openBrowserRequest)

goToRequest = _reflection.GeneratedProtocolMessageType('goToRequest', (_message.Message,), {
  'DESCRIPTOR' : _GOTOREQUEST,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:goToRequest)
  })
_sym_db.RegisterMessage(goToRequest)

inputTextRequest = _reflection.GeneratedProtocolMessageType('inputTextRequest', (_message.Message,), {
  'DESCRIPTOR' : _INPUTTEXTREQUEST,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:inputTextRequest)
  })
_sym_db.RegisterMessage(inputTextRequest)

getDomPropertyRequest = _reflection.GeneratedProtocolMessageType('getDomPropertyRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDOMPROPERTYREQUEST,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:getDomPropertyRequest)
  })
_sym_db.RegisterMessage(getDomPropertyRequest)

selectorRequest = _reflection.GeneratedProtocolMessageType('selectorRequest', (_message.Message,), {
  'DESCRIPTOR' : _SELECTORREQUEST,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:selectorRequest)
  })
_sym_db.RegisterMessage(selectorRequest)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {

  'Empty' : _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_EMPTY,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.Empty)
    })
  ,

  'String' : _reflection.GeneratedProtocolMessageType('String', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_STRING,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.String)
    })
  ,

  'Bool' : _reflection.GeneratedProtocolMessageType('Bool', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_BOOL,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.Bool)
    })
  ,
  'DESCRIPTOR' : _RESPONSE,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:Response)
  })
_sym_db.RegisterMessage(Response)
_sym_db.RegisterMessage(Response.Empty)
_sym_db.RegisterMessage(Response.String)
_sym_db.RegisterMessage(Response.Bool)



_PLAYWRIGHT = _descriptor.ServiceDescriptor(
  name='Playwright',
  full_name='Playwright',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=402,
  serialized_end=1114,
  methods=[
  _descriptor.MethodDescriptor(
    name='Screenshot',
    full_name='Playwright.Screenshot',
    index=0,
    containing_service=None,
    input_type=_SCREENSHOTREQUEST,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='OpenBrowser',
    full_name='Playwright.OpenBrowser',
    index=1,
    containing_service=None,
    input_type=_OPENBROWSERREQUEST,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CloseBrowser',
    full_name='Playwright.CloseBrowser',
    index=2,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GoTo',
    full_name='Playwright.GoTo',
    index=3,
    containing_service=None,
    input_type=_GOTOREQUEST,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetTitle',
    full_name='Playwright.GetTitle',
    index=4,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_RESPONSE_STRING,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='InputText',
    full_name='Playwright.InputText',
    index=5,
    containing_service=None,
    input_type=_INPUTTEXTREQUEST,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetDomProperty',
    full_name='Playwright.GetDomProperty',
    index=6,
    containing_service=None,
    input_type=_GETDOMPROPERTYREQUEST,
    output_type=_RESPONSE_STRING,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetBoolProperty',
    full_name='Playwright.GetBoolProperty',
    index=7,
    containing_service=None,
    input_type=_GETDOMPROPERTYREQUEST,
    output_type=_RESPONSE_BOOL,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetTextContent',
    full_name='Playwright.GetTextContent',
    index=8,
    containing_service=None,
    input_type=_SELECTORREQUEST,
    output_type=_RESPONSE_STRING,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetUrl',
    full_name='Playwright.GetUrl',
    index=9,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_RESPONSE_STRING,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ClickButton',
    full_name='Playwright.ClickButton',
    index=10,
    containing_service=None,
    input_type=_SELECTORREQUEST,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CheckCheckbox',
    full_name='Playwright.CheckCheckbox',
    index=11,
    containing_service=None,
    input_type=_SELECTORREQUEST,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UncheckCheckbox',
    full_name='Playwright.UncheckCheckbox',
    index=12,
    containing_service=None,
    input_type=_SELECTORREQUEST,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Health',
    full_name='Playwright.Health',
    index=13,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_RESPONSE_STRING,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_PLAYWRIGHT)

DESCRIPTOR.services_by_name['Playwright'] = _PLAYWRIGHT

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from Browser.generated import playwright_pb2 as playwright__pb2


class PlaywrightStub(object):
    """Missing associated documentation comment in .proto file"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Screenshot = channel.unary_unary(
                '/Playwright/Screenshot',
                request_serializer=playwright__pb2.screenshotRequest.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.OpenBrowser = channel.unary_unary(
                '/Playwright/OpenBrowser',
                request_serializer=playwright__pb2.openBrowserRequest.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.CloseBrowser = channel.unary_unary(
                '/Playwright/CloseBrowser',
                request_serializer=playwright__pb2.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GoTo = channel.unary_unary(
                '/Playwright/GoTo',
                request_serializer=playwright__pb2.goToRequest.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GetTitle = channel.unary_unary(
                '/Playwright/GetTitle',
                request_serializer=playwright__pb2.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.InputText = channel.unary_unary(
                '/Playwright/InputText',
                request_serializer=playwright__pb2.inputTextRequest.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GetDomProperty = channel.unary_unary(
                '/Playwright/GetDomProperty',
                request_serializer=playwright__pb2.getDomPropertyRequest.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.GetBoolProperty = channel.unary_unary(
                '/Playwright/GetBoolProperty',
                request_serializer=playwright__pb2.getDomPropertyRequest.SerializeToString,
                response_deserializer=playwright__pb2.Response.Bool.FromString,
                )
        self.GetTextContent = channel.unary_unary(
                '/Playwright/GetTextContent',
                request_serializer=playwright__pb2.selectorRequest.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.GetUrl = channel.unary_unary(
                '/Playwright/GetUrl',
                request_serializer=playwright__pb2.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.ClickButton = channel.unary_unary(
                '/Playwright/ClickButton',
                request_serializer=playwright__pb2.selectorRequest.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.CheckCheckbox = channel.unary_unary(
                '/Playwright/CheckCheckbox',
                request_serializer=playwright__pb2.selectorRequest.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.UncheckCheckbox = channel.unary_unary(
                '/Playwright/UncheckCheckbox',
                request_serializer=playwright__pb2.selectorRequest.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.Health = channel.unary_unary(
                '/Playwright/Health',
                request_serializer=playwright__pb2.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )


class PlaywrightServicer(object):
    """Missing associated documentation comment in .proto file"""

    def Screenshot(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def OpenBrowser(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseBrowser(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GoTo(self, request, context):
        """Opens the url in currently open Playwright page 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTitle(self, request, context):
        """Gets title of currently open Playwright page 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def InputText(self, request, context):
        """Wraps playwrights page.fill to input text into input specified with selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDomProperty(self, request, context):
        """Gets the DOM property 'property' of selector specified element 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBoolProperty(self, request, context):
        """Gets the boolean DOM property 'property' of selector specified element 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTextContent(self, request, context):
        """Wraps playwrights page.textContent, returns textcontent of element by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetUrl(self, request, context):
        """*Returns current playwright page url
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClickButton(self, request, context):
        """Clicks button specified by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CheckCheckbox(self, request, context):
        """Checks checkbox specified by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UncheckCheckbox(self, request, context):
        """Unchecks checkbox specified by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Health(self, request, context):
        """Health check endpoint for the service 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PlaywrightServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Screenshot': grpc.unary_unary_rpc_method_handler(
                    servicer.Screenshot,
                    request_deserializer=playwright__pb2.screenshotRequest.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'OpenBrowser': grpc.unary_unary_rpc_method_handler(
                    servicer.OpenBrowser,
                    request_deserializer=playwright__pb2.openBrowserRequest.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'CloseBrowser': grpc.unary_unary_rpc_method_handler(
                    servicer.CloseBrowser,
                    request_deserializer=playwright__pb2.Empty.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GoTo': grpc.unary_unary_rpc_method_handler(
                    servicer.GoTo,
                    request_deserializer=playwright__pb2.goToRequest.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GetTitle': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTitle,
                    request_deserializer=playwright__pb2.Empty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'InputText': grpc.unary_unary_rpc_method_handler(
                    servicer.InputText,
                    request_deserializer=playwright__pb2.inputTextRequest.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GetDomProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDomProperty,
                    request_deserializer=playwright__pb2.getDomPropertyRequest.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'GetBoolProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBoolProperty,
                    request_deserializer=playwright__pb2.getDomPropertyRequest.FromString,
                    response_serializer=playwright__pb2.Response.Bool.SerializeToString,
            ),
            'GetTextContent': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTextContent,
                    request_deserializer=playwright__pb2.selectorRequest.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'GetUrl': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUrl,
                    request_deserializer=playwright__pb2.Empty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'ClickButton': grpc.unary_unary_rpc_method_handler(
                    servicer.ClickButton,
                    request_deserializer=playwright__pb2.selectorRequest.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'CheckCheckbox': grpc.unary_unary_rpc_method_handler(
                    servicer.CheckCheckbox,
                    request_deserializer=playwright__pb2.selectorRequest.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'UncheckCheckbox': grpc.unary_unary_rpc_method_handler(
                    servicer.UncheckCheckbox,
                    request_deserializer=playwright__pb2.selectorRequest.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'Health': grpc.unary_unary_rpc_method_handler(
                    servicer.Health,
                    request_deserializer=playwright__pb2.Empty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Playwright', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Playwright(object):
    """Missing associated documentation comment in .proto file"""

    @staticmethod
    def Screenshot(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Screenshot',
            playwright__pb2.screenshotRequest.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def OpenBrowser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/OpenBrowser',
            playwright__pb2.openBrowserRequest.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CloseBrowser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/CloseBrowser',
            playwright__pb2.Empty.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GoTo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GoTo',
            playwright__pb2.goToRequest.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTitle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetTitle',
            playwright__pb2.Empty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def InputText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/InputText',
            playwright__pb2.inputTextRequest.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDomProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetDomProperty',
            playwright__pb2.getDomPropertyRequest.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetBoolProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetBoolProperty',
            playwright__pb2.getDomPropertyRequest.SerializeToString,
            playwright__pb2.Response.Bool.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTextContent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetTextContent',
            playwright__pb2.selectorRequest.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetUrl(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetUrl',
            playwright__pb2.Empty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClickButton(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/ClickButton',
            playwright__pb2.selectorRequest.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CheckCheckbox(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/CheckCheckbox',
            playwright__pb2.selectorRequest.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UncheckCheckbox(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/UncheckCheckbox',
            playwright__pb2.selectorRequest.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Health(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Health',
            playwright__pb2.Empty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)



from typing import Optional

from robot.api import logger  # type: ignore
from robotlibcore import keyword  # type: ignore

from ..generated import playwright_pb2


class Control:
    def __init__(self, library):
        self.library = library

    @property
    def playwright(self):
        return self.library.playwright

    @keyword
    def open_browser(self, browser="Chrome", url=None):
        """Opens a new browser instance to the optional ``url``.
        The ``browser`` argument specifies which browser to use. The
        supported browsers are listed in the table below. The browser names
        are case-insensitive and some browsers have multiple supported names.
        |    = Browser =    |        = Name(s) =        |
        | Firefox           | firefox                   |
        | Google Chrome     | chrome                    |
        | WebKit            | webkit                    |

        """
        browser_ = browser.lower().strip()
        if browser_ not in self.library.SUPPORTED_BROWSERS:
            raise ValueError(
                f"{browser} is not supported, "
                f'it should be one of: {", ".join(self.library.SUPPORTED_BROWSERS)}'
            )
        with self.playwright.grpc_channel() as stub:
            response = stub.OpenBrowser(
                playwright_pb2.openBrowserRequest(url=url or "", browser=browser_)
            )
            logger.info(response.log)

    @keyword
    def close_browser(self):
        """Closes the current browser."""
        with self.playwright.grpc_channel() as stub:
            response = stub.CloseBrowser(playwright_pb2.Empty())
            logger.info(response.log)

    @keyword
    def get_url(self):
        """Returns curent URL."""
        with self.playwright.grpc_channel() as stub:
            response = stub.GetUrl(playwright_pb2.Empty())
            logger.info(response.log)
            return response.body

    @keyword
    def go_to(self, url: str):
        """Navigates the current browser tab to the provided ``url``."""
        with self.playwright.grpc_channel() as stub:
            response = stub.GoTo(playwright_pb2.goToRequest(url=url))
            logger.info(response.log)

    @keyword
    def take_page_screenshot(self, path: Optional[str] = None):
        """ Take screenshot of current browser state and saves it to path.
            By default saves to robot output dir.
        """
        if path is None:
            path = self.library.get_screenshot_path
        logger.info(f"Taking screenshot into ${path}")
        with self.playwright.grpc_channel() as stub:
            response = stub.Screenshot(playwright_pb2.screenshotRequest(path=path))
            logger.info(response.log)



import contextlib
import functools
import os
import socket
from subprocess import Popen, STDOUT
import time

import grpc  # type: ignore
from robot.api import logger  # type: ignore
from robot.libraries.BuiltIn import BuiltIn  # type: ignore

from Browser.generated.playwright_pb2 import Empty
import Browser.generated.playwright_pb2_grpc as playwright_pb2_grpc


class Playwright:
    """A wrapper for communicating with nodejs Playwirght process."""

    @property
    def outputdir(self):
        return BuiltIn().get_variable_value("${OUTPUTDIR}")

    @functools.cached_property
    def _playwright_process(self) -> Popen:
        process = self.start_playwright()
        self.wait_until_server_up()
        return process

    def start_playwright(self):
        workdir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "wrapper")
        playwright_script = os.path.join(workdir, "index.js")
        logfile = open(os.path.join(self.outputdir, "playwright-log.txt"), "w",)
        port = str(self.find_free_port())
        env = dict(os.environ)
        env["PORT"] = port
        logger.info(f"Starting Browser process {playwright_script} using port {port}")
        self.port = port
        return Popen(
            ["node", playwright_script],
            shell=False,
            cwd=workdir,
            env=env,
            stdout=logfile,
            stderr=STDOUT,
        )

    def wait_until_server_up(self):
        for i in range(30):
            with grpc.insecure_channel(f"localhost:{self.port}") as channel:
                try:
                    stub = playwright_pb2_grpc.PlaywrightStub(channel)
                    response = stub.Health(Empty())
                    logger.info(
                        f"Connected to the playwright process at port {self.port}: {response}"
                    )
                    return
                except grpc.RpcError as err:
                    logger.info(err)
                    time.sleep(0.1)
        raise RuntimeError(
            f"Could not connect to the playwright process at port {self.port}"
        )

    @contextlib.contextmanager
    def grpc_channel(self):
        """Yields a PlayWrightstub on a newly initialized channel

        Acts as a context manager, so channel is closed automatically when control returns.
        """
        returncode = self._playwright_process.poll()
        if returncode is not None:
            raise ConnectionError(
                "Playwright process has been terminated with code {}".format(returncode)
            )
        channel = grpc.insecure_channel(f"localhost:{self.port}")
        yield playwright_pb2_grpc.PlaywrightStub(channel)
        channel.close()

    def close(self):
        logger.debug("Closing Playwright process")
        self._playwright_process.kill()
        logger.debug("Playwright process killed")

    def find_free_port(self) -> str:
        with contextlib.closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
            s.bind(("", 0))
            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            return s.getsockname()[1]



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages  # type: ignore

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

packages = \
    find_packages(exclude=["utest", "atest"])

package_data = \
    {'': ['*'], 'Browser': ['wrapper/*', 'wrapper/generated/*']}

install_requires = \
    open("requirements.txt").readlines()

setup_kwargs = {
    'name': 'robotframework-browser',
    'version': '0.2.2',
    'description': '',
    'long_description': long_description,
    'long_description_content_type': 'text/markdown',
    'author': 'Mikko Korpela',
    'author_email': 'mikko.korpela@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/MarketSquare/robotframework-browser',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': {'console_scripts': ['rfbrowser=Browser.entry:run']},
    'python_requires': '>=3.8,<4.0',
    'classifiers': [
        "Development Status :: 1 - Planning",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Testing",
        "Framework :: Robot Framework",
        "Framework :: Robot Framework :: Library"
    ]
}


setup(**setup_kwargs)



from enum import Enum

from robot.api import logger  # type: ignore
from robotlibcore import keyword  # type: ignore

from ..generated import playwright_pb2


class Validation:
    def __init__(self, library):
        self.library = library

    @property
    def playwright(self):
        return self.library.playwright

    @keyword
    def location_should_be(self, url: str):
        """ Verifies that the current URL is exactly ``url``. """
        with self.playwright.grpc_channel() as stub:
            page_url = stub.GetUrl(playwright_pb2.Empty()).body
            if url != page_url:
                message = "URL should be `{}`  but was `{}`".format(url, page_url)
                raise AssertionError(message)

    @keyword
    def textfield_value_should_be(self, selector: str, expected: str):
        """Verifies text field ``selector`` has exactly text ``expected``. """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetDomProperty(
                playwright_pb2.getDomPropertyRequest(
                    selector=selector, property="value"
                )
            )
            logger.info(response.log)
            if response.body != expected:
                message = "Textfield {} content should be {} but was `{}`".format(
                    selector, expected, response.body
                )
                raise AssertionError(message)

    @keyword
    def title_should_be(self, title: str):
        """ Verifies that the current page title equals ``title`` """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetTitle(playwright_pb2.Empty())
            logger.info(response.log)
            if response.body != title:
                message = "Title should be {} but was `{}`".format(title, response.body)
                raise AssertionError(message)

    @keyword
    def page_should_contain(self, text: str):
        """Verifies that current page contains ``text``. """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetTextContent(
                playwright_pb2.selectorRequest(selector="text=" + text)
            )
            logger.info(response.log)
            if response.body != text:
                message = "No element with text `{}` on page".format(text)
                raise AssertionError(message)

    class CheckboxState(Enum):
        checked = True
        unchecked = False

    @keyword
    def checkbox_should_be(self, selector: str, expected: CheckboxState):
        """ Verifies that checkbox ``selector`` is in state ``expected`` """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetBoolProperty(
                playwright_pb2.getDomPropertyRequest(
                    selector=selector, property="checked"
                )
            )
            logger.info(response.log)
            if response.body != expected.value:
                message = "Checkbox `{}` should be `{}` but was `{}`".format(
                    selector, expected, response.body
                )
                raise AssertionError(message)
